__all__ = [
    "__author__",
    "__copyright__",
    "__email__",
    "__license__",
    "__summary__",
    "__title__",
    "__uri__",
    "__version__",
]

__title__ = "ssdeep"
__summary__ = "Python wrapper for the ssdeep library"
__uri__ = "http://github.com/DinoTools/python-ssdeep"

__version__ = "3.1"

__author__ = "PhiBo (DinoTools)"
__email__ = ""

__license__ = "LGPLv3+"
__copyright__ = "Copyright 2014 %s" % __author__
